/**************************************************************************//**
 * @file     init_hook.c
 * @brief    初始化钩子
 * @detail   在主函数中首先被调用，适用于初始化时钟、IO、寄存器等工作。
 ******************************************************************************/

#include "..\System\os_link.h"
#define BAUD1  115200		// 串口1波特率
#define TM1    (65536 - (MCUCFG_SYSCLK * 1000000UL) / 4 / BAUD1) // 串口1用定时器1

void init_hook(void)
{
	AUXR	=	0x00;		// 允许访问内部xdata
	P_SW2	=	EAXFR;	// 允许访问扩展RAM区特殊功能寄存器
	
	TMOD = 0x00;		// 定时器0、1、2、3、4为16位自动重装模式
	AUXR |= 0x44;		// 定时器1、2为1T模式，串口1用定时器1
	SM0 = 0;
	SM1 = 1;				// 串口1工作模式为模式1
	P_SW1 |= 0xC0;	// 串口1转移至_4
	T1L =	(u8)TM1;
	T1H =	(u8)(TM1 >> 8);
	TR1 = 1;
	IPH	|= PSH;
	PS = 1;
	TI = 1;
	#if SYSCFG_DEBUGGING == 1
	ES = 1;
	REN = 1;
	#endif
	/* PendSV用INT0 */
	IPH&= ~PX0H;
	PX0 =	0;
}

#if SYSCFG_DEBUGGING == 1
void UART1_Isr(void) interrupt 4 using 3
{
	if(TI)
	{
		TI = 0;
		if(*vDebugSendPtr) SBUF = *vDebugSendPtr++;
		else iDebugSend(0);
	}
	if(RI)
	{
		RI = 0;
		if(vDebugRecvPtr - vDebugRecvBuff < OS_DEBUGRECVBUFFSIZE - 1) *vDebugRecvPtr++ = SBUF;
		iDebugRecv(1);
	}
}
#endif
